/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1074;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1271;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1800;
import net.minecraft.class_1836;
import net.minecraft.class_1838;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2378;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2874;
import net.minecraft.class_2960;
import net.minecraft.util.*;
import reborncore.client.hud.StackInfoElement;
import reborncore.common.util.ChatUtils;
import techreborn.TechReborn;
import techreborn.init.TRContent;
import techreborn.utils.MessageIDs;

import javax.annotation.Nullable;
import java.util.List;

public class ItemFrequencyTransmitter extends class_1792 {

	public ItemFrequencyTransmitter() {
		super(new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP).method_7889(1));
		this.method_7863(new class_2960("techreborn", "coords"), new class_1800() {
			@Override
			@Environment(EnvType.CLIENT)
			public float call(class_1799 stack, @Nullable class_1937 worldIn, @Nullable class_1309 entityIn) {
				if (!stack.method_7960() && stack.method_7985() && stack.method_7969() != null && stack.method_7969().method_10545("x")
						&& stack.method_7969().method_10545("y") && stack.method_7969().method_10545("z") && stack.method_7969().method_10545("dim")) {
					return 1.0F;
				}
				return 0.0F;
			}
		});
	}

	@Override
	public class_1269 method_7884(class_1838 context) {
		class_1937 world = context.method_8045();
		class_2338 pos = context.method_8037();
		class_1799 stack = context.method_8041();

		stack.method_7980(new class_2487());
		stack.method_7969().method_10569("x", pos.method_10263());
		stack.method_7969().method_10569("y", pos.method_10264());
		stack.method_7969().method_10569("z", pos.method_10260());
		stack.method_7969().method_10569("dim", world.method_8597().method_12460().method_12484());

		if (!world.field_9236) {
			ChatUtils.sendNoSpamMessages(MessageIDs.freqTransmitterID, new class_2585(
				class_124.field_1080 + class_1074.method_4662("techreborn.message.setTo") + " X: " +
					class_124.field_1065 + pos.method_10263() +
					class_124.field_1080 + " Y: " +
					class_124.field_1065 + pos.method_10264() +
					class_124.field_1080 + " Z: " +
					class_124.field_1065 + pos.method_10260() +
					class_124.field_1080 + " " + class_1074.method_4662("techreborn.message.in") + " " +
					class_124.field_1065 + getDimName(world.method_8597().method_12460())
					+ " (" + getDimName(world.method_8597().method_12460()) + ")"));
		}
		return class_1269.field_5812;
	}

	@Override
	public class_1271<class_1799> method_7836(class_1937 world, class_1657 player,
	                                                class_1268 hand) {
		class_1799 stack = player.method_5998(hand);
		if (player.method_5715()) {
			stack.method_7980(null);
			if (!world.field_9236) {
				ChatUtils.sendNoSpamMessages(MessageIDs.freqTransmitterID, new class_2585(
					class_124.field_1080 + class_1074.method_4662("techreborn.message.coordsHaveBeen") + " "
						+ class_124.field_1065 + class_1074.method_4662("techreborn.message.cleared")));
			}
		}

		return new class_1271<>(class_1269.field_5812, stack);
	}

	@Environment(EnvType.CLIENT)
	@Override
	public void method_7851(class_1799 stack, @Nullable class_1937 worldIn, List<class_2561> tooltip, class_1836 flagIn) {
		if (stack.method_7985() && stack.method_7969() != null && stack.method_7969().method_10545("x") && stack.method_7969().method_10545("y") && stack.method_7969().method_10545("z") && stack.method_7969().method_10545("dim")) {
			int x = stack.method_7969().method_10550("x");
			int y = stack.method_7969().method_10550("y");
			int z = stack.method_7969().method_10550("z");
			int dim = stack.method_7969().method_10550("dim");

			tooltip.add(new class_2585(class_124.field_1080 + "X: " + class_124.field_1065 + x));
			tooltip.add(new class_2585(class_124.field_1080 + "Y: " + class_124.field_1065 + y));
			tooltip.add(new class_2585(class_124.field_1080 + "Z: " + class_124.field_1065 + z));
			tooltip.add(new class_2585(class_124.field_1063 + getDimName(class_2378.field_11155.method_10200(dim)).toString()));

		} else {
			tooltip.add(new class_2585(class_124.field_1080 + class_1074.method_4662("techreborn.message.noCoordsSet")));
		}
	}

	private static class_2960 getDimName(class_2874 type){
		return class_2378.field_11155.method_10221(type);
	}

	public static class StackInfoFreqTransmitter extends StackInfoElement {
		public StackInfoFreqTransmitter() {
			super(TRContent.FREQUENCY_TRANSMITTER);
		}

		@Override
		public String getText(class_1799 stack) {
			String text = "";
			class_124 gold = class_124.field_1065;
			class_124 grey = class_124.field_1080;
			if (stack.method_7909() instanceof ItemFrequencyTransmitter) {
				if (stack.method_7985() && stack.method_7969() != null && stack.method_7969().method_10545("x") && stack.method_7969().method_10545("y") && stack.method_7969().method_10545("z") && stack.method_7969().method_10545("dim")) {
					int coordX = stack.method_7969().method_10550("x");
					int coordY = stack.method_7969().method_10550("y");
					int coordZ = stack.method_7969().method_10550("z");
					int coordDim = stack.method_7969().method_10550("dim");
					text = grey + "X: " + gold + coordX + grey + " Y: " + gold + coordY + grey + " Z: " + gold + coordZ + grey + " Dim: " + gold + getDimName(class_2378.field_11155.method_10200(coordDim)).toString() + " (" + coordDim + ")";
				} else {
					text = grey + class_1074.method_4662("techreborn.message.noCoordsSet");
				}
			}
			return text;
		}
	}
}
